/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/05/26 08:58:55 $
		$Revision: 1.15 $
*/

#include "stdafx.h"
using namespace std;

namespace p2p {
	void list::_load_p2b(istream &stream) {
		char buf[7];
		unsigned char version;
		if(
			!stream.read(buf, sizeof(buf)) ||
			memcmp(buf, "\xFF\xFF\xFF\xFFP2B", 7) ||
			!stream.read((char*)&version, sizeof(version))
		) throw p2p_error("invalid p2b stream");

		if(version==1 || version==2) {
			unsigned int start, end;
			
			string name;
			while(getline(stream, name, '\0')) {
				if(
					!stream.read((char*)&start, sizeof(start)) ||
					!stream.read((char*)&end, sizeof(end))
				) throw p2p_error("invalid p2b stream");
				
				range r;

				r.name.reserve(name.length());
				if(version==1) { // P2B v1 is expected to be ISO-8859-1 encoded.
					copy(name.begin(), name.end(), back_inserter(r.name));
				}
				else if(version==2) utf8_wchar(name, r.name);

				boost::trim(r.name);

				start=ntohl(start);
				end=ntohl(end);

				r.start=min(start,end);
				r.end=max(start,end);

				this->insert(r);
			}
		}
		else if(version==3) {
			unsigned int namecount;
			if(!stream.read((char*)&namecount, sizeof(namecount))) throw p2p_error("invalid p2b stream: name count expected");
			namecount=ntohl(namecount);

			boost::scoped_array<wstring> names(new wstring[namecount]);

			for(unsigned int i=0; i<namecount; i++) {
				string name;
				if(!getline(stream, name, '\0')) throw p2p_error("invalid p2b stream: name expected");

				utf8_wchar(name, names[i]);
			}

			unsigned int rangecount;
			if(!stream.read((char*)&rangecount, sizeof(rangecount))) throw p2p_error("invalid p2b stream: range count expected");
			rangecount=ntohl(rangecount);

			unsigned int name, start, end;

			for(unsigned int i=0; i<rangecount; i++) {
				if(
					!stream.read((char*)&name, sizeof(name)) ||
					!stream.read((char*)&start, sizeof(start)) ||
					!stream.read((char*)&end, sizeof(end))
				) throw p2p_error("invalid p2b stream: range expected");

				name=ntohl(name);
				start=ntohl(start);
				end=ntohl(end);

				range r(names[name], min(start,end), max(start,end));
				boost::trim(r.name);

				this->insert(r);
			}
		}
		else throw p2p_error("unknown p2b version");
	}

	void list::_save_p2b(ostream &stream) const {
		stream.write("\xFF\xFF\xFF\xFFP2B\x03", 8);

		map<wstring,unsigned int> names;
		unsigned int i=0;
		
		{
			vector<wstring> namevec;

			for(list::const_iterator iter=this->begin(); iter!=this->end(); iter++)
				if(names.find(iter->name)==names.end()) {
					names[iter->name]=i++;
					namevec.push_back(iter->name);
				}

			i=htonl(i);
			stream.write((const char*)&i, sizeof(i));

			for(vector<wstring>::size_type j=0; j<namevec.size(); j++) {
				string name;
				wchar_utf8(namevec[j], name);
				stream.write(name.c_str(), (streamsize)name.size()+1);
			}
		}

		i=htonl((unsigned int)this->size());
		stream.write((const char*)&i, sizeof(i));

		for(list::const_iterator iter=this->begin(); iter!=this->end(); iter++) {
			unsigned int name=htonl(names[iter->name]);
			unsigned int start=htonl(iter->start.ipl);
			unsigned int end=htonl(iter->end.ipl);

			stream.write((const char*)&name, sizeof(name));
			stream.write((const char*)&start, sizeof(start));
			stream.write((const char*)&end, sizeof(end));
		}
	}
}
